program PrintHelp;

{$R+}

const
  HelpPageWidth = 74;
  HelpPageLength = 20;
  MaxSectionIndex = 20;
  NoneIndex = 255;
  HelpFileName = 'C:Layout.Hlp';
  PrinterPageLength = 68;
  PageGap = 4;
  Emphasize = #27'E';
  Cancel = #27'F';

type
  HelpLineType = string [HelpPageWidth];
  HelpPageType = record
                   PageName: HelpLineType;
                   PreviousPageIndex, NextPageIndex,
                   PreviousParagraphIndex, NextParagraphIndex,
                   CurrentParagraphOriginIndex: byte;
                   HelpLines: array [1..HelpPageLength] of HelpLineType
                 end;
  PagePointersType = record
                       PreviousPageIndex, NextPageIndex,
                       PreviousParagraphIndex, NextParagraphIndex,
                       CurrentParagraphOriginIndex: integer;
                       StartOfSection, StartOfParagraph: Boolean;
                     end;

PROCEDURE PrintHelpFile;

  var
    HelpFile: file of HelpPageType;
    HelpPage: HelpPageType;
    SectionIndex,
    LineNumber: integer;

  procedure GetHelpPage (Index: integer);

    procedure NewPage;

      var i: integer;
          c: char;

      begin
        LineNumber := LineNumber + 1;
        if LineNumber = PrinterPageLength then
          begin
            for i := 1 to PageGap do Writeln (Lst);
            LineNumber := 0;
            if KeyPressed then
              begin
                Read (Kbd, c);
                repeat
                  Write ('Abort printing? (Y/N)');
                  Read (c);
                until (UpCase (c) = 'Y') or (UpCase (c) = 'N');
                if UpCase (c) = 'Y' then Halt
              end
          end
      end {NewPage};

    procedure PrintHelpPage ({read only} var HelpPage: HelpPageType);

      var HelpPageLineNumber: integer;

      begin
        if HelpPage. CurrentParagraphOriginIndex = FilePos (HelpFile) - 1 then
          begin
            Writeln (Lst);
            NewPage;
            Writeln (Lst, Emphasize, HelpPage. PageName, Cancel);
            NewPage
          end;
        for HelpPageLineNumber := 1 to HelpPageLength do
          begin
            Writeln (Lst, HelpPage. HelpLines [HelpPageLineNumber]);
            NewPage
          end
      end {PrintHelpPage};

    begin {GetHelpPage}
      Seek (HelpFile, Index);
      Read (HelpFile, HelpPage);
      PrintHelpPage (HelpPage);
    end {GetHelpPage};

  begin
    LineNumber := 0;
    Assign (HelpFile, HelpFileName);
    Reset (HelpFile);
    for SectionIndex := 0 to MaxSectionIndex do
      begin
        GetHelpPage (SectionIndex);
        with HelpPage do
          while NextPageIndex <> NoneIndex do GetHelpPage (NextPageIndex)
      end
  end {PrintHelpFile};

begin
  PrintHelpFile
end.
